import { ILogger, IPargv } from 'pargv';
export declare type BeforeRender = {
    (template?: ITemplate, done?: {
        ();
    });
};
export declare type NodeCallback = {
    (err?: Error, data?: any);
};
export declare type InjectCallback = {
    (line: string, data: string, done: {
        (line: string, data: string);
    });
};
export interface IMetadata {
    component?: string;
    [key: string]: any;
}
export interface IPartials {
    [key: string]: string | ITemplate;
}
export interface ITemplate {
    name?: string;
    ext?: string;
    suffix?: string;
    casing?: 'lower' | 'upper' | 'camel' | 'title' | 'capitalize';
    path?: string;
    pathNormalized?: string;
    metadata?: IMetadata;
    body?: string;
    beforeRender?: BeforeRender;
    outputPath?: string;
    raw?: string;
    partials?: IPartials;
}
export interface ITemplates {
    [key: string]: ITemplate;
}
export interface IMustrOptions {
    configDir?: string;
    outputDir?: string;
    templateExt?: string;
}
export interface IMustr {
    cwd: string;
    pargv: IPargv;
    log: ILogger;
    registerPath: string;
    templatesPath: string;
    outputPath: string;
    loaded: boolean;
    templatesGlob: string[];
    templates: ITemplates;
    options: IMustrOptions;
    help(): void;
    init(force?: boolean): void;
    load(): IMustr;
    normalize(template: string): any;
    register(name: string, template: string, partials?: string | string[] | BeforeRender, action?: BeforeRender): IMustr;
    generate(name?: string, output?: string | IMetadata, options?: IMetadata): ITemplate;
    render(path: string | ITemplate, body?: string, data?: {
        [key: string]: any;
    }, partials?: {
        [key: string]: any;
    }): void;
    inject(filename: string, search: string | RegExp, value: string | string[] | InjectCallback, replace: boolean | NodeCallback, done?: NodeCallback): void;
    transformCase(str: string, to: string): string;
}
