
import { Mustr } from './index';
import { resolve } from 'path';

let mustrConfig;

const mustrPath = resolve(process.cwd(), 'mustr.json');

// Try to load the mustr.json config if exists.
try {
  mustrConfig = require(mustrPath);
}
catch (ex) {
  mustrConfig = {};
}

// Create Mustr instance.
const mu = new Mustr(mustrConfig);

// Just add some spacing.
console.log();

// Handler for generating templates.
function generate() {

  if (!mustrConfig || !mustrConfig.configDir)
    mu.log.warn('invalid or missing Mustr config, do you need to call mu init?').exit();

  // Generate the template.
  const template = mu.generate();

  // If was registered with an action call it.
  if (template.beforeRender) {
    template.beforeRender(template, () => {
      mu.render.call(mu, template);
    });
  }

  // Otherwise don't wait just render.
  else {
    mu.render.call(mu, template);
  }
}

// Bind CLI methods.
mu.pargv
  .action('help', 'h', mu.help.bind(mu))
  .action('init', 'i', mu.init.bind(mu))
  .action('generate', 'g', generate)
  .action('*', () => {
    mu.log
      .warn('no command selected, did you mean to call "mu help"?')
      .write()
      .exit();
  })
  .parse();




