import { ILogger, IPargv } from 'pargv';

export type BeforeRender = { (template?: ITemplate, done?: { () }) };

export type NodeCallback = { (err?: Error, data?: any) };

export type InjectCallback = { (line: string, data: string, done: { (line: string, data: string) }) }

export interface IMetadata {
    component?: string;   // the component name generated from command line.
    [key: string]: any;   // other metadata passed in template or on rendering.
}

export interface IPartials {
    [key: string]: string | ITemplate;
}

export interface ITemplate {

    name?: string;            // name of template used ONLY when passing static templates.
    ext?: string;             // extension to use when rendered.
    suffix?: string;          // suffix to be added to the component name.
    casing?: 'lower' | 'upper' | 'camel' | 'title' | 'capitalize';  // component name case.
    path?: string;            // the path to the template if not static defined template.
    pathNormalized?: string;  // the path without template extension.
    metadata?: IMetadata;     // metadata for template rendering.
    body?: string;            // the template body.
    beforeRender?: BeforeRender;  // the callback before rendering/compiling template.
    outputPath?: string;      // the path to output the template to.
    raw?: string;             // a static template that has not been parsed.
    partials?: IPartials;

}

export interface ITemplates {
    [key: string]: ITemplate;
}

export interface IMustrOptions {
    configDir?: string;     // the Muster dir containing templates and register.
    outputDir?: string;     // the output directory when templates are rendered.
    templateExt?: string;   // the extension for templates usually .tpl.
}

export interface IMustr {

    cwd: string;
    pargv: IPargv;
    log: ILogger;
    registerPath: string;
    templatesPath: string;
    outputPath: string;
    loaded: boolean;
    templatesGlob: string[];
    templates: ITemplates;
    options: IMustrOptions;

    help(): void;
    init(force?: boolean): void;
    load(): IMustr;
    normalize(template: string): any;
    register(name: string, template: string, partials?: string | string[] | BeforeRender, action?: BeforeRender): IMustr;
    generate(name?: string, output?: string | IMetadata, options?: IMetadata): ITemplate;
    render(path: string | ITemplate, body?: string, data?: { [key: string]: any }, partials?: { [key: string]: any }): void;
    inject(filename: string, search: string | RegExp, value: string | string[] | InjectCallback, replace: boolean | NodeCallback, done?: NodeCallback): void;
    transformCase(str: string, to: string): string;

}