import * as chai from 'chai';
import * as mocha from 'mocha';
import { parse } from 'path';
import * as fs from 'fs-extra';

const expect = chai.expect;
const should = chai.should;
const assert = chai.assert;

import { Mustr } from '../src';

const mu = new Mustr({
    configDir: './src/blueprints',
    outputDir: './test/output'
});

// Load up configs.
mu.load();

let template;

describe('Mustr', () => {

    before((done) => {

        fs.removeSync('./test/output');

        mu.register('DB', 'example.class', 'example.author');
        template = mu.generate('DB', 'examples/db.tsx', {
            author: 'Origin1 Technologies'
        });
        done();

    });

    it('should register a template named "DB".', () => {
        assert.deepProperty(mu.templates, 'DB');
    });

    it('should generate the template "DB".', () => {

        const parsed = parse(template.outputPath);

        // Ensure component name and casing.
        assert.equal(template.metadata.component, 'DbExample');

        // Ensure suffix and extension in output name.
        assert.equal(parsed.base, 'db.example.tsx');

    });

    it('should output the generated DbExample template.', () => {

        mu.render(template);
        assert.equal(fs.existsSync(template.outputPath), true);

    });

    it('it should inject value into rendered template.', (done) => {

        mu.inject('./test/output/examples/db.example.tsx', '// INJECT', "import * as _ from 'lodash';", true, () => {
            done();
        });

    });

    after((done) => {

        done();

    });

});